<?php

/*

 HtPasswd File Reader with Abilities

 -- Copyright --
 Copyright 2004, by James McDuffie (ttq3sxl02@sneakemail.com)
 Portions (C) 2005, by Roman Mamedov (http://rm.pp.ru/?en.contact)
 
 -- License --
 GNU GPL

 -- Description --

 This class implements reading of a .htpasswd file. It simply reads in
 three field from the file separated by the ':' character:
    username:password:abilities
 The password is encrypted in the usual Apache htpasswd file way. The
 abilites are a comma separated list of actions the user can perform.
 
 -- Example Usage --

 $ht_obj = new HtPasswd('/tmp/.htpasswd');
 $fred_password = $ht_obj->GetUserPassword('fred');
 $fred_can_read = $ht_obj->UserHasAbility('fred', 'read');
 
 -- Change log --

 v0.4 -- 3-1-2005 -- JM
 * Added capability to write object to disk
 * Added setting of passwords into object
 
 v0.3 -- 26.02.2005 -- RM
 * fixed a couple of bugs

 v0.2 -- 25.02.2005 -- RM
 * $pwd_file_contents are now loaded with file() instead of
   file_get_contents(), so it also works on PHP older than 4.3
 * $abilities_str is now trim()'med, so it works correctly with
   htpasswd which has windows newlines
 * greatly simplified UserHasAbility(), without hurting
   its functionality

*/

define(PRESENCE_USER_DATA_CLASS, '0.1');
define(PRESENCE_USERS_CLASS, '0.1');

Class Presence_User_Data {
	var $password;
	var $contact;
	var $contactID;
	var $abilities;

	function Presence_User_Data() {
		$this->password = "";
		$this->contact = "";
		$this->contactID = "";
		$this->abilities = "";

	}
	
	function Set_User_Data($pwd, $cnt, $cntID, $abl) {
		$this->password = $pwd;
		$this->contact = $cnt;
		$this->contactID = $cntID;
		$this->abilities = $abl;
	}
	
	function Set_Password($password) {
		$this->password = crypt($password);
	}
	
	function Set_Contact($contact) {
		$this->contact = $contact;
	}
	
	function Set_ContactID($contactID) {
		$this->contactID = $contactID;
	}
	
	function Set_Abilities($abilites_array) {
		$this->abilities = $abilites_array;
	}
	
	function Get_Password() {
		return $this->password;
	}
	
	function Get_Contact() {
		return $this->contact;
	}
	
	function Get_ContactID() {
		return $this->contactID;
	}
	
	function Get_Abilities() {
		return $this->abilities;
	}
	
	function Add_Ability($ability) {
		if (!in_array($ability, $this->abilities)) {
			array_push($this->abilities, $ability);
		}
	}
	
	function Del_Ability($ability) {
		$ability_loc = array_search($ability, $this->abilities);
		unset ($this->abilities[$ability_loc]);
	}
}
Class Presence_Users {
	var $users = array ();
	var $users_data_file;
	# user_passwords - An array hash where the key is the username and value 
	# the password
	#
	# user_abilites - An array hash where the key is the username and the value
	# is an array of ability strings
	##	var $user_passwords = array ();
	##	var $user_abilities = array ();

	// Below are the HtPasswd specific methods. These do not need to be 
	// implemented persay if you create a new UserInfoObj class. But
	// you still probably need a constructor for your new class, but you
	// don't necessarily need one that requires a filename

	function Presence_Users($users_data_file) {
		$this->users_data_file = $users_data_file;
		$this->Read_User_Data_File();
	}

	// Never call the below methods directly, instead use the generic
	// method names defined later in this class

	function Read_User_Data_File() {
		if (!file_exists($this->users_data_file)) {
			trigger_error("userauth htpasswd file does not exist!");
		}
		@ $pwd_file_contents = file($this->users_data_file);
		foreach ($pwd_file_contents as $file_line) {
			# Skip lines not beginning with letter or numbers
			if (preg_match('/^[\w\d]+/', $file_line)) {
				#if($file_line != "") {
				list ($username, $password, $contact, $contactID, $abilities_str) = preg_split("/\s*:\s*/", $file_line);
				@ $abilities = preg_split("/\s*,\s*/", trim($abilities_str));
				$this->users[$username] = new Presence_User_Data();
				$this->users[$username]->Set_User_Data($password, $contact, $contactID, $abilities);
			}
		}
	}

	function Write_User_Data_File() {
		// Make a backup of the password file just in case somethign goes wrong
		copy($this->users_data_file, $this->users_data_file.".bak");
		if (!is_writable($this->users_data_file)) {
			trigger_error("userauth htpasswd file is not writable");
		}
		$pw_file_handle = fopen($this->users_data_file, 'w');
		// For each user write an entry to disk
		foreach ($this->Get_All_Users() as $curr_username) {
			$curr_password = $this->users[$curr_username]->Get_Password();
			$curr_contact = $this->users[$curr_username]->Get_Contact();
			$curr_contactID = $this->users[$curr_username]->Get_ContactID();
			$curr_abilities = $this->users[$curr_username]->Get_Abilities();
			$user_out_line = $curr_username.":".$curr_password.":".$curr_contact.":".$curr_contactID.":".implode(",", $curr_abilities)."\r\n";
			//echo $user_out_line."</br>";
			if (fwrite($pw_file_handle, $user_out_line) === FALSE) {
				trigger_error("userauth htpasswd file can not be written to");
			}
		}
		fclose($pw_file_handle);
	}

	// If you wish to implement a new UserInfoObj type object then
	// Implementing the same methods as below will allow you to easily
	// use your new class instead of this one.

	function Publish_Changes() {
		// This function has a generic name so that anyone implementing a new
		// UserInfoObj class is not bound to the notion of passwords and abilities
		// being stored in a file. Instead they could be stored in a db
		$this->Write_User_Data_File();
	}

	function Any_User_Has_Ability($ability) {
		foreach ($this->Get_All_Users() as $curr_username) {
			if (@ in_array($ability, $this->users[$curr_username]->Get_Abilities())) {
				return true;
			}
		}
		return false;
	}

	function User_Has_Ability($username, $ability) {
		
		return @ in_array($ability, $this->users[$username]->Get_Abilities());
	}

	function Does_User_Exist($username) {
		return isset ($this->users[$username]);
	}

	function Add_User_Ability($username, $ability) {
		$this->users[$username]->Add_Ability($ability);
	}

	function Del_User_Ability($username, $ability) {
		$this->users[$username]->Del_Ability($ability);
	}

	function Get_User_Abilities($username) {
		return $this->users[$username]->Get_Abilities();
	}

	function Set_User_Abilities($username, $abilities_array) {
		$this->users[$username]->Set_Abilities($abilities_array);
	}

	function Get_User_Password($username) {
		return $this->users[$username]->Get_Password();
	}

	function Set_User_Password($username, $newpassword) {
		$this->users[$username]->Set_Password($newpassword);
	}
	
    function Set_User_Contact($username, $contact){
    	$this->users[$username]->Set_Contact($contact);
    }
    function Get_User_Contact($username){
    	return $this->users[$username]->Get_Contact();
    }
    
    function Set_User_ContactID($username, $contactID){
    	$this->users[$username]->Set_ContactID($contactID);
    }
    function Get_User_ContactID($username){
    	return $this->users[$username]->Get_ContactID();
    }
    
	function Get_All_Users() {
		return array_keys($this->users);
	}

	function Add_User($username) {
		$this->users[$username] = new Presence_User_Data();
	}

	function Del_User($username) {
		unset ($this->users[$username]);
	}

}
?>



